rm(list=ls(all=TRUE))

library("foreign")
library("Hmisc")
library("haven")
library("readr")
library("tidyr")
library("dplyr")
library("ggplot2")
library("gridExtra")
library("ltm")
library("pryr")
library("car")
library("xtable")
library("ebal")
library("mgcv")
library("survey")
library("splines")
source("ResBal.R")

tmp_fun_a <- function(x){ifelse(x %in% -seq(1, 5), NA, x)}
tmp_fun_b <- compose(as.numeric, as.character)
percentile_rank <- function(x){rank(x, na.last = "keep")/sum(!is.na(x))}

Mode <- function(x) {
  x <- x[complete.cases(x)]
  ux <- unique(x)
  ux[which.max(tabulate(match(x, ux)))]
}

wtd_rank <- function (x, weights = NULL, normwt = FALSE, na.rm = TRUE){
  if (!length(weights))
    return(rank(x, na.last = if (na.rm) NA else TRUE))
  tab <- Hmisc::wtd.table(x, weights, normwt = normwt, na.rm = na.rm)
  freqs <- tab$sum.of.weights
  r <- cumsum(freqs) - 0.5 * (freqs - 1)
  stats::approx(tab$x, r, xout = x, rule = 2)$y
}

load("nlsy79_samples.RData")

origin_bs <- ns(nlsy79_ed_b$origin, df = 3)

res <- function(y, x, w) {
  if(identical(sort(unique(y)), c(0, 1))){
    return(y - glm.fit(y = y, x = x, weights = w, family = binomial("logit"))[["fitted.values"]])
    } else return(lm.wfit(y = y, x = x, w = w)[["residuals"]])
  }

nlsy79_ed_b <- cbind(nlsy79_ed_b, origin_bs) %>%
  `names<-`(c(names(.)[1:(ncol(.)-3)], paste("origin_bs", 1:3, sep = ""))) %>%
  mutate(female = as.numeric(sex == 2)) %>%
  # mutate_each(funs(res = res(., x = cbind(1, origin) , w = weights)), c(test_score:num_sibs, female)) %>%
  mutate_each(funs(res = res(., x = cbind(1, origin_bs1, origin_bs2, origin_bs3) , w = weights)), c(test_score:num_sibs, female)) %>%
  mutate_each(funs(origin_bs1 = `*`(., origin_bs1)), test_score_res:female_res)  %>%
  mutate_each(funs(origin_bs2 = `*`(., origin_bs2)), test_score_res:female_res)  %>%
  mutate_each(funs(origin_bs3 = `*`(., origin_bs3)), test_score_res:female_res)  %>%
  mutate_each(funs(college = `*`(., college)), test_score_res:female_res) %>%
  mutate_each(funs(origin_bs1 = `*`(., origin_bs1)), c(test_score_res_college:female_res_college)) %>%
  mutate_each(funs(origin_bs2 = `*`(., origin_bs2)), c(test_score_res_college:female_res_college)) %>%
  mutate_each(funs(origin_bs3 = `*`(., origin_bs3)), c(test_score_res_college:female_res_college))

# tmp <- with(nlsy79_ed_b, lm(college ~ origin_bs1 + origin_bs2 + origin_bs3, weights = weights))
# summary(tmp$residuals)

# nlsy79_ed_b <- nlsy79_ed_b %>%
#   mutate(female = as.numeric(sex == 2)) %>%
#   mutate(college_res =  tmp$residuals) %>%
#   mutate_each(funs(college_res = `*`(., college_res)), c(test_score:num_sibs, female, origin_bs1:origin_bs3)) 

summary(nlsy79_ed_b)

# residual balancing

C <- cbind(1, subset(nlsy79_ed_b, select = test_score_res:female_res_college)) %>% as.matrix()
Q <- nlsy79_ed_b$weights
M <- c(sum(nlsy79_ed_b$weights), rep(0, ncol(C)-1))

RB <- ResBal(M, C, Q, tol = 0.001)
nlsy79_ed_b$wt_ebal_res <- RB$W

nlsy79_ed2_b <- nlsy79_ed_b

summary(RB$W)

# bootstrap msm

# coef_holder <- matrix(NA, 1000, 4)
# 
# for (i in 1:1000){
# 
#   nlsy79_ed_b_boot <- sample_n(nlsy79_ed_b, size = nrow(nlsy79_ed_b), replace = TRUE)
# 
#   C <- cbind(1, subset(nlsy79_ed_b_boot, select = c(test_score_res:female_res_college))) %>% as.matrix()
#   Q <- nlsy79_ed_b_boot$weights
#   M <- c(sum(nlsy79_ed_b_boot$weights), rep(0, ncol(C)-1))
# 
#   RB <- ResBal(M, C, Q, tol = 0.001, print.level = 0)
#   nlsy79_ed_b_boot$wt_ebal_res <- RB$W
# 
#   msm_b_ebal <- lm(dest ~ origin + college + origin*college,
#                    weights = wt_ebal_res, data = nlsy79_ed_b_boot)
# 
#   coef_holder[i, ] <- coef(msm_b_ebal)
# }
# 
# apply(coef_holder, 2, sd)

# data 

college_grad <- outer(nlsy79_ed2_b$college, c(FALSE, TRUE), `==`)

nlsy79_ed3_b <- nlsy79_ed2_b %>% 
  mutate(wt_adjusted_ebal = wt_ebal_res) %>%
  group_by(college) %>%
  mutate(wt_adjusted_ebal = wt_adjusted_ebal/sum(wt_adjusted_ebal)*sum(weights)) %>%
  ungroup()

# balance in E(X|O) estimated as the slope of lm(X ~ O)

lm_coef <- function(x, y, w) lm.wfit(x, y, w)[["coefficients"]][2]

conditional <- nlsy79_ed3_b %>% group_by(college) %>%
  summarise_each(funs(lm_coef(x = cbind(1, origin), ., w = weights)),
                 c(test_score:num_sibs, female)) %>% `[`(, -1) %>% t()

overall <- nlsy79_ed3_b %>% summarise_each(funs(lm_coef(x = cbind(1, origin), ., w = weights)),
                                           c(test_score:num_sibs, female)) %>% t()

reweighted <- nlsy79_ed3_b %>% group_by(college) %>%
  summarise_each(funs(lm_coef(x = cbind(1, origin), ., w = wt_adjusted_ebal)),
                 c(test_score:num_sibs, female)) %>% `[`(, -1) %>% t()

balance_b <- cbind(overall, conditional, reweighted)

# prep

data_new <- expand.grid(college = c(FALSE, TRUE), origin = seq(0, 1, 0.01))
origin_bs_simp <- ns(data_new$origin, knots = attr(origin_bs, "knots"))
colnames(origin_bs_simp) <- paste("origin_bs", 1:ncol(origin_bs_simp), sep = "")
data_new <- cbind(data_new, origin_bs_simp)

formula1 <- dest ~ origin + college + origin*college
formula2 <- dest ~ origin_bs1 + origin_bs2 + origin_bs3 + college +
  origin_bs1*college + origin_bs2*college + origin_bs3*college

# check balance plots

var2bal <- c("female", "test_score", "educ_exp", "father", "hispanic", "black", "urban", "educ_mom", "num_sibs")

formulas <- lapply(var2bal, compose(as.formula, paste), " ~ origin_bs1 + origin_bs2 + origin_bs3 + college +
                   origin_bs1*college + origin_bs2*college + origin_bs3*college")

nlsy79_ed3_b_augmented <- nlsy79_ed3_b

nlsy79_ed3_b_augmented <- nlsy79_ed3_b %>% filter(college==TRUE) %>%
  mutate(college = FALSE) %>% rbind(nlsy79_ed3_b)

design_naive <- svydesign(ids = ~1, weights = ~ weights, data = nlsy79_ed3_b_augmented)
design_ebal <- svydesign(ids = ~1, weights = ~ wt_adjusted_ebal, data = nlsy79_ed3_b_augmented)

svyglm_naive <- partial(svyglm, design = design_naive)
svyglm_naive_logit <- partial(svyglm, family = binomial("logit"), design = design_naive)
svyglms_naive <- list(svyglm_naive_logit, svyglm_naive, svyglm_naive, svyglm_naive_logit, svyglm_naive_logit,
                      svyglm_naive_logit, svyglm_naive_logit, svyglm_naive, svyglm_naive)

svyglm_ebal <- partial(svyglm, design = design_ebal)
svyglm_ebal_logit <- partial(svyglm, family = binomial("logit"), design = design_ebal)
svyglms_ebal <- list(svyglm_ebal_logit, svyglm_ebal, svyglm_ebal, svyglm_ebal_logit, svyglm_ebal_logit,
                     svyglm_ebal_logit, svyglm_ebal_logit, svyglm_ebal, svyglm_ebal)

cov_b_before_ebal <- Map(function(f, x) f(x), svyglms_naive, formulas)
cov_b_after_ebal <- Map(function(f, x) f(x), svyglms_ebal, formulas)

plot_b_before_ebal <- lapply(cov_b_before_ebal, function(x) cbind(data_new, predict(x, data_new, type = "response")))
plot_b_after_ebal <- lapply(cov_b_after_ebal, function(x) cbind(data_new, predict(x, data_new, type = "response")))

var_labels <- c("Female", "Black", "Hispanic", "Mother's Education", "Father Presence",
                "Number of Siblings", "Urban Residence", "Adjusted AFQT Score", "Educational Expectation") 
var_levels <- c("female", "black", "hispanic", "educ_mom", "father", "num_sibs", "urban", "test_score", "educ_exp")

figureB1_input <- Reduce(rbind, plot_b_before_ebal) %>%
  dplyr::select(college, origin, fitted = response, se = SE) %>%
  mutate(cov = factor(rep(var2bal, each = 202),
                      levels = var_levels, labels = var_labels),
         Education = factor(college, levels = c(F, T),
                            labels = c("Full Sample",
                                       "College Graduates")))

ggplot(figureB1_input, aes(x = origin, y = fitted)) +
  geom_line(aes(linetype = Education, col = Education), size = 1.1) +
  geom_ribbon(aes(ymin = fitted - 1.645*se, ymax = fitted + 1.645*se, fill = Education), alpha = 0.2) +
  facet_wrap(~cov, scales = "free_y", ncol = 3) +
  labs(x = "Parental Income Rank", y = "Conditonal Means of Covariates") +
  theme_minimal(base_size = 18) +
  theme(legend.position = "bottom") +
  guides(col = guide_legend(title = "", nrow = 1),
         fill = guide_legend(title = "", nrow = 1),
         linetype = guide_legend(title = "", nrow = 1)) 

figureB2_input <- Reduce(rbind, plot_b_after_ebal) %>%
  dplyr::select(college, origin, fitted = response, se = SE) %>%
  mutate(cov = factor(rep(var2bal, each = 202),
                      levels = var_levels, labels = var_labels),
         Education = factor(college, levels = c(F, T),
                            labels = c("Full Sample",
                                       "College Graduates")))

ggplot(figureB2_input, aes(x = origin, y = fitted)) +
  geom_line(aes(linetype = Education, col = Education), size = 1.1) +
  geom_ribbon(aes(ymin = fitted - 1.645*se, ymax = fitted + 1.645*se, fill = Education), alpha = 0.2) +
  facet_wrap( ~ cov, scales = "free_y", ncol = 3) +
  labs(x = "Parental Income Rank", y = "Conditonal Means of Covariates") +
  theme_minimal(base_size = 18) +
  theme(legend.position = "bottom") +
  guides(col = guide_legend(title = "", nrow = 1),
         fill = guide_legend(title = "", nrow = 1),
         linetype = guide_legend(title = "", nrow = 1)) 

# msm naive

design_naive <- svydesign(ids = ~1, weights = ~ weights, data = nlsy79_ed3_b)
msm_b_naive <- svyglm(formula1, design = design_naive)
msm_b_naive_plot <- cbind(data_new, predict(msm_b_naive, newdata = data_new))

msm2_b_naive <- svyglm(formula2, design = design_naive)
msm2_b_naive_plot <- cbind(data_new, predict(msm2_b_naive, newdata = data_new))

summary(msm_b_naive)

# with ebal weights

design_ebal <- svydesign(ids = ~1, weights = ~ wt_adjusted_ebal, data = nlsy79_ed3_b)
msm_b_ebal <- svyglm(formula1, design = design_ebal)
msm_b_ebal_plot <- cbind(data_new, predict(msm_b_ebal, newdata = data_new))

msm2_b_ebal <- svyglm(formula2, design = design_ebal)
msm2_b_ebal_plot <- cbind(data_new, predict(msm2_b_ebal, newdata = data_new))

summary(msm_b_ebal)

# figure5_input <- rbind(msm2_b_naive_plot, msm2_b_ebal_plot) %>%
#   dplyr::select(college, origin, fitted = link, se = SE) %>%
#   mutate(method = factor(rep(c("before", "after"), each = 202),
#                          levels = c("before", "after"),
#                          labels = c("Before Reweighting", "After Reweighting")),
#          Education = factor(college, levels = c(F, T), labels = c("Non-College Graduates", "College Graduates")))
# 
# ggplot(figure5_input, aes(x = origin, y = fitted)) +
#   geom_line(aes(linetype = Education, col = Education), size = 1.1) +
#   geom_ribbon(aes(ymin = fitted - 1.645*se, ymax = fitted + 1.645*se, fill = Education), alpha = 0.2) +
#   facet_wrap(~method, scales = "fixed", ncol = 2) +
#   labs(x = "Parental Income Rank", y = "Adult Income Rank") +
#   theme_minimal(base_size = 20) +
#   theme(legend.position = "bottom")  +
#   guides(col = guide_legend(title = "", nrow = 1),
#          fill = guide_legend(title = "", nrow = 1),
#          linetype = guide_legend(title = "", nrow = 1)) 

# RWR

# formula3 <-  dest ~ college * (origin + female_res + test_score_res +
#   educ_exp_res + father_res + hispanic_res + black_res + urban_res + educ_mom_res + num_sibs_res)
# 
# msm_b_rwr <- lm(formula3, data = nlsy79_ed3_b)
# 
# summary(msm_b_rwr)

# sensitivity analysis

varbal2 <-  c("female", "black", "hispanic", "educ_mom", "father", "num_sibs", "urban", "test_score", "educ_exp")

z_formulas <- lapply(varbal2, compose(as.formula, paste), " ~ origin + college + I(origin*college)")

dest_formulas <- lapply(varbal2, compose(as.formula, partial(paste, "dest ~ origin + college + I(origin*college) +")))

sensitivity <- function(z_formula, dest_formula){
  z_model <- svyglm(z_formula, design = design_naive)
  dest_model <- svyglm(dest_formula, design = design_ebal)
  z_coef <- coef(z_model)[4]
  dest_coef <- coef(dest_model)[5]
  bias <- z_coef * dest_coef
  c(z_coef = z_coef, dest_coef = dest_coef, bias = bias)
}

sensitivity_out <- Map(sensitivity, z_formulas, dest_formulas) %>%
  simplify2array() %>% t() %>% data.frame() %>%
  `names<-`(c("alpha3", "gamma", "bias")) %>%
  mutate(adj_est = msm_b_ebal$coef[4] - bias) %>%
  `rownames<-`(varbal2)

save.image("results_full.RData")

