rm(list=ls(all=TRUE))

library("foreign")
library("Hmisc")
library("haven")
library("readr")
library("tidyr")
library("dplyr")
library("ggplot2")
library("gridExtra")
library("ltm")
library("pryr")
library("car")
library("xtable")
library("ebal")
library("twang")
library("mgcv")
library("survey")
library("splines")
source("ResBal.R")

tmp_fun_a <- function(x){ifelse(x %in% -seq(1, 5), NA, x)}
tmp_fun_b <- compose(as.numeric, as.character)
percentile_rank <- function(x){rank(x, na.last = "keep")/sum(!is.na(x))}

Mode <- function(x) {
  x <- x[complete.cases(x)]
  ux <- unique(x)
  ux[which.max(tabulate(match(x, ux)))]
}

wtd_rank <- function (x, weights = NULL, normwt = FALSE, na.rm = TRUE){
  if (!length(weights))
    return(rank(x, na.last = if (na.rm) NA else TRUE))
  tab <- Hmisc::wtd.table(x, weights, normwt = normwt, na.rm = na.rm)
  freqs <- tab$sum.of.weights
  r <- cumsum(freqs) - 0.5 * (freqs - 1)
  stats::approx(tab$x, r, xout = x, rule = 2)$y
}

load("nlsy79_samples.RData")

nlsy79_male_ed_b <- nlsy79_male_ed_b %>% filter(advanced == FALSE) %>%
  filter(tfinc_origin > 0, tfinc_dest_b > 0) %>%
  mutate(origin = log(tfinc_origin), dest = log(tfinc_dest_b))

origin_bs <- ns(nlsy79_male_ed_b$origin, df = 3)

res <- function(y, x, w) {
  if(identical(sort(unique(y)), c(0, 1))){
    return(y - glm.fit(y = y, x = x, weights = w, family = binomial("logit"))[["fitted.values"]])
  } else return(lm.wfit(y = y, x = x, w = w)[["residuals"]])
}

nlsy79_male_ed_b <- cbind(nlsy79_male_ed_b, origin_bs) %>%
  `names<-`(c(names(.)[1:(ncol(.)-3)], paste("origin_bs", 1:3, sep = ""))) %>%
  mutate_each(funs(res = res(., x = cbind(1, origin_bs1, origin_bs2, origin_bs3) , w = weights)), c(test_score:num_sibs)) %>%
  mutate_each(funs(origin_bs1 = `*`(., origin_bs1)), test_score_res:num_sibs_res)  %>%
  mutate_each(funs(origin_bs2 = `*`(., origin_bs2)), test_score_res:num_sibs_res)  %>%
  mutate_each(funs(origin_bs3 = `*`(., origin_bs3)), test_score_res:num_sibs_res)  %>%
  mutate_each(funs(college = `*`(., college)), test_score_res:num_sibs_res) %>%
  mutate_each(funs(origin = `*`(., origin)), c(test_score_res_college:num_sibs_res_college)) %>%
  mutate_each(funs(origin_bs1 = `*`(., origin_bs1)), c(test_score_res_college:num_sibs_res_college)) %>%
  mutate_each(funs(origin_bs2 = `*`(., origin_bs2)), c(test_score_res_college:num_sibs_res_college)) %>%
  mutate_each(funs(origin_bs3 = `*`(., origin_bs3)), c(test_score_res_college:num_sibs_res_college))

# residual balance 

C <- cbind(1, subset(nlsy79_male_ed_b, select = test_score_res:num_sibs_res_college)) %>% as.matrix()
Q <- nlsy79_male_ed_b$weights
M <- c(sum(nlsy79_male_ed_b$weights), rep(0, ncol(C)-1))

RB <- ResBal(M, C, Q, tol = 0.01)
nlsy79_male_ed_b$wt_ebal_res <- RB$W

nlsy79_male_ed2_b <- nlsy79_male_ed_b

# data 

college_grad <- outer(nlsy79_male_ed2_b$college, c(FALSE, TRUE), `==`)

nlsy79_male_ed3_b <- nlsy79_male_ed2_b %>% 
  mutate(wt_adjusted_ebal = wt_ebal_res) %>%
  group_by(college) %>%
  mutate(wt_adjusted_ebal = wt_adjusted_ebal/sum(wt_adjusted_ebal)*sum(weights)) %>%
  ungroup()

# balance in E(X|O) estimated as the slope of lm(X ~ O)

lm_coef <- function(x, y, w) lm.wfit(x, y, w)[["coefficients"]][2]

conditional <- nlsy79_male_ed3_b %>% group_by(college) %>%
  summarise_each(funs(lm_coef(x = cbind(1, origin), ., w = weights)),
                 c(test_score:num_sibs)) %>% `[`(, -1) %>% t()

overall <- nlsy79_male_ed3_b %>% summarise_each(funs(lm_coef(x = cbind(1, origin), ., w = weights)),
                                                  c(test_score:num_sibs)) %>% t()

reweighted <- nlsy79_male_ed3_b %>% group_by(college) %>%
  summarise_each(funs(lm_coef(x = cbind(1, origin), ., w = wt_adjusted_ebal)),
                 c(test_score:num_sibs)) %>% `[`(, -1) %>% t()

balance_b <- cbind(overall, conditional, reweighted)

# prep

formula1 <- dest ~ origin + college + origin*college
formula2 <- dest ~ origin_bs1 + origin_bs2 + origin_bs3 + college +
  origin_bs1*college + origin_bs2*college + origin_bs3*college

var_labels <- c("Black", "Hispanic", "Mother's Education", "Father Presence",
                "Number of Siblings", "Urban Residence", "Adjusted AFQT Score", "Educational Expectation") 
var_levels <- c("black", "hispanic", "educ_mom", "father", "num_sibs", "urban", "test_score", "educ_exp")

# msm naive

design_naive <- svydesign(ids = ~1, weights = ~ weights, data = nlsy79_male_ed3_b)
msm_b_naive_male <- svyglm(formula1, design = design_naive)

summary(msm_b_naive_male)

# with ebal weights

design_ebal <- svydesign(ids = ~1, weights = ~ wt_adjusted_ebal, data = nlsy79_male_ed3_b)
msm_b_ebal_male <- svyglm(formula1, design = design_ebal)

summary(msm_b_ebal_male)

save.image("results_male_noadv_IGE.RData")

