rm(list=ls(all=TRUE))

library("foreign")
library("Hmisc")
library("haven")
library("readr")
library("tidyr")
library("dplyr")
library("ggplot2")
library("gridExtra")
library("ltm")
library("pryr")
library("car")
library("xtable")
library("ebal")
library("mgcv")
library("survey")
library("splines")
source("01_star.R")

load("results_male.RData")
load("results_female.RData")
load("results_full.RData")

var2bal <- c("female", "test_score", "educ_exp", "father", "hispanic", "black", "urban", "educ_mom", "num_sibs")

var_labels <- c("Female", "Black", "Hispanic", "Mother's Education", "Father Presence",
                "Number of Siblings", "Urban Residence", "Adjusted AFQT Score", "Educational Expectation") 
var_levels <- c("female", "black", "hispanic", "educ_mom", "father", "num_sibs", "urban", "test_score", "educ_exp")

table(nlsy79_ed3_b$college3)

nrow(nlsy79_ed3_b)
nrow(nlsy79_male_ed3_b)
nrow(nlsy79_female_ed3_b)

# table 6

rownames(sensitivity_out) <- var_labels

write.csv(sensitivity_out, "table5sensitivity.csv")

# figure_tab6 <- data.frame(x = c("None", var_labels),
#            y = c(msm_b_ebal$coef[4], sensitivity_out$adj_est)) %>%
#   mutate(x = factor(x, levels = rev(x)),
#          z = c(0, rep(1, 9)))
#
# ggplot(figure_tab6, aes(x, y)) +
#   geom_col(aes(fill = factor(z))) +
#   labs(x = "Strength of Unobserved Selection",
#        y = expression(paste("Adjusted Estimate of ", beta))) +
#   scale_fill_manual(values = c("darkorange2", "white")) + 
#   theme_minimal(base_size = 18) +
#   theme(legend.position = "none") +
#   coord_flip()
# 
# ggsave("figure_tab6a.tiff", height = 6, width = 9)
# 
# ggplot(figure_tab6, aes(x, y)) +
#   geom_col(aes(fill = factor(z))) +
#   labs(x = "Strength of Unobserved Selection",
#        y = expression(paste("Adjusted Estimate of ", beta))) +
#   scale_fill_manual(values = c("darkorange2", "skyblue2")) + 
#   theme_minimal(base_size = 18) +
#   theme(legend.position = "none") +
#   coord_flip()
#   
# ggsave("figure_tab6b.tiff", width = 9, height = 6)

# table 1

table1 <- nlsy79_ed3_b %>% group_by(college3) %>%
  summarise_at(c("origin", "dest", var_levels), funs(weighted.mean(., w = weights))) %>%
  t() %>% `[`(-1, ) %>% data.frame() %>%
  `names<-`(c("non-graduates", "graduates from nonselective colleges", "graduates from selective colleges"))

write.csv(table1, "table1.csv")

# table 2

coef <- list(msm_b_naive, msm_b_ebal,
             msm_b_naive_male, msm_b_ebal_male,
             msm_b_naive_female, msm_b_ebal_female) %>%
  lapply(summary) %>% lapply(coef) %>%  lapply(`[`, , 1) %>% simplify2array()

se <- list(msm_b_naive, msm_b_ebal,
           msm_b_naive_male, msm_b_ebal_male,
           msm_b_naive_female, msm_b_ebal_female) %>%
  lapply(summary) %>% lapply(coef) %>% lapply(`[`, , 2) %>% simplify2array()

comb(coef, se) %>% write.csv("table2.csv", row.names = FALSE)

# write.csv(coef, "table2coef.csv")
# write.csv(se, "table2se.csv")
#
# list(msm_b_naive, msm_b_ebal,
#      msm_b_naive_male, msm_b_ebal_male,
#      msm_b_naive_female, msm_b_ebal_female) %>%
#   lapply(summary)

# old table 1 & 2

# balance_a <- balance_a[var_levels, ]
# rownames(balance_a) <- var_labels
# colnames(balance_a) <- c("overall", "no college", "college", "no college reweighted", "college reweighted")
# 
# balance_b <- balance_b[var_levels, ]
# rownames(balance_b) <- var_labels
# colnames(balance_b) <- c("overall", "no college", "college", "no college reweighted", "college reweighted")
# 
# write.csv(balance_a, "table1.csv")
# write.csv(balance_b, "table2.csv")

# figure 3: balance before reweighting

figure3_input <- Reduce(rbind, plot_b_before_ebal) %>%
  dplyr::select(college, origin, fitted = response, se = SE) %>%
  mutate(cov = factor(rep(var2bal, each = 202),
                      levels = var_levels, labels = var_labels),
         Education = factor(college, levels = c(F, T), labels = c("Full Sample", "College Graduates")))

ggplot(figure3_input, aes(x = origin, y = fitted)) +
  geom_line(aes(linetype = Education, col = Education), size = 1.1) +
  geom_ribbon(aes(ymin = fitted - 1.645*se, ymax = fitted + 1.645*se, fill = Education), alpha = 0.2) +
  facet_wrap( ~ cov, scales = "free_y", ncol = 3) +
  labs(x = "Parental Income Rank", y = "Conditonal Means of Covariates") +
  theme_minimal(base_size = 18) +
  theme(legend.position = "bottom") +
  guides(col = guide_legend(title = "", nrow = 1),
         fill = guide_legend(title = "", nrow = 1),
         linetype = guide_legend(title = "", nrow = 1)) 

ggsave("figure3.tiff", width = 9, height = 9)
# ggsave("figure3slides.tiff", width = 9, height = 7)

# figure 4: balance after reweighting

figure4_input <- Reduce(rbind, plot_b_after_ebal) %>%
  dplyr::select(college, origin, fitted = response, se = SE) %>%
  mutate(cov = factor(rep(var2bal, each = 202),
                      levels = var_levels, labels = var_labels),
         Education = factor(college, levels = c(F, T), labels = c("Full Sample", "College Graduates")))

ggplot(figure4_input, aes(x = origin, y = fitted)) +
  geom_line(aes(linetype = Education, col = Education), size = 1.1) +
  geom_ribbon(aes(ymin = fitted - 1.645*se, ymax = fitted + 1.645*se, fill = Education), alpha = 0.2) +
  facet_wrap(~cov, scales = "free_y", ncol = 3) +
  labs(x = "Parental Income Rank", y = "Conditonal Means of Covariates") +
  theme_minimal(base_size = 18) +
  theme(legend.position = "bottom") +
  guides(col = guide_legend(title = "", nrow = 1),
         fill = guide_legend(title = "", nrow = 1),
         linetype = guide_legend(title = "", nrow = 1)) 

ggsave("figure4.tiff", width = 9, height = 9)
# ggsave("figure4slides.tiff", width = 9, height = 7)

# figure 4 old: histogram of weights

# nlsy79_ed3_b %>% group_by(college) %>% summarise(sum(weights))
# nlsy79_ed3_b %>% group_by(college) %>% summarise(sum(wt_adjusted_ebal))
# 
# figure5_input <- with(nlsy79_ed3_b, rbind(cbind(weights, measure = 0), cbind(wt_adjusted_ebal, measure = 1))) %>%
#   tbl_df() %>% mutate(measure = factor(measure, levels = c(0, 1), labels = c("Original Weights", "Adjusted Weights")))
# 
# ggplot(figure5_input, aes(x = weights)) +
#   geom_histogram(aes(fill = measure)) +
#   facet_wrap(~measure) +
#   labs(x = "Weight", y = "Frequency") +
#   # scale_fill_manual(values = c("darkorange", "deepskyblue3"))+
#   scale_x_log10() +
#   theme_minimal(base_size = 20) +
#   theme(legend.position = "none") 
# 
# ggsave("figure5.tiff", width = 9, height = 6)
# ggsave("figure5slides.tiff", width = 9, height = 6)

# figure 5: conditional and controlled mobility

figure5_input <- rbind(msm_b_naive_plot, msm_b_ebal_plot) %>%
  dplyr::select(college, origin, fitted = link, se = SE) %>%
  mutate(method = factor(rep(c("before", "after"), each = 202),
                         levels = c("before", "after"),
                         labels = c("Before Reweighting", "After Reweighting")),
         Education = factor(college, levels = c(F, T), labels = c("Non-College Graduates", "College Graduates")))

ggplot(figure5_input, aes(x = origin, y = fitted)) +
  geom_line(aes(linetype = Education, col = Education), size = 1.1) +
  geom_ribbon(aes(ymin = fitted - 1.645*se, ymax = fitted + 1.645*se, fill = Education), alpha = 0.2) +
  facet_wrap(~method, scales = "fixed", ncol = 2) +
  labs(x = "Parental Income Rank", y = "Adult Income Rank") +
  theme_minimal(base_size = 20) +
  theme(legend.position = "bottom")  +
  guides(col = guide_legend(title = "", nrow = 1),
         fill = guide_legend(title = "", nrow = 1),
         linetype = guide_legend(title = "", nrow = 1)) 

ggsave("figure5.tiff", width = 9, height = 6)
# ggsave("figure5slides_b.tiff", width = 9, height = 6)

ggplot(figure5_input %>% mutate(fitted = ifelse(method == "After Reweighting", NA, fitted)), aes(x = origin, y = fitted)) +
  geom_line(aes(linetype = Education, col = Education), size = 1.1) +
  geom_ribbon(aes(ymin = fitted - 1.645*se, ymax = fitted + 1.645*se, fill = Education), alpha = 0.2) +
  facet_wrap(~method, scales = "fixed", ncol = 2) +
  labs(x = "Parental Income Rank", y = "Adult Income Rank") +
  theme_minimal(base_size = 20) +
  theme(legend.position = "bottom")  +
  guides(col = guide_legend(title = "", nrow = 1),
         fill = guide_legend(title = "", nrow = 1),
         linetype = guide_legend(title = "", nrow = 1)) 

# ggsave("figure5slides_a.tiff", width = 9, height = 6)

# 3-category results # 

load("results_male_edu3.RData")
load("results_female_edu3.RData")
load("results_full_edu3.RData")

var_labels <- c("Female", "Black", "Hispanic", "Mother's Education", "Father Presence",
                "Number of Siblings", "Urban Residence", "Adjusted AFQT Score", "Educational Expectation") 
var_levels <- c("female", "black", "hispanic", "educ_mom", "father", "num_sibs", "urban", "test_score", "educ_exp")

# table 3

coef <- list(msm_b_naive, msm_b_ebal,
             msm_b_naive_male, msm_b_ebal_male,
             msm_b_naive_female, msm_b_ebal_female) %>%
  lapply(summary) %>% lapply(coef) %>%  lapply(`[`, , 1) %>% simplify2array()

se <- list(msm_b_naive, msm_b_ebal,
           msm_b_naive_male, msm_b_ebal_male,
           msm_b_naive_female, msm_b_ebal_female) %>%
  lapply(summary) %>% lapply(coef) %>% lapply(`[`, , 2) %>% simplify2array()

comb(coef, se) %>% write.csv("table3.csv", row.names = FALSE)

# list(msm_b_naive, msm_b_ebal,
#      msm_b_naive_male, msm_b_ebal_male,
#      msm_b_naive_female, msm_b_ebal_female) %>%
#   lapply(summary) 
# 
# write.csv(coef, "table3coef.csv")
# write.csv(se, "table3se.csv")

# old table 1 & 2

# balance_a <- balance_a[var_levels, ]
# rownames(balance_a) <- var_labels
# colnames(balance_a) <- c("overall", "no college", "college", "no college reweighted", "college reweighted")
# 
# balance_b <- balance_b[var_levels, ]
# rownames(balance_b) <- var_labels
# colnames(balance_b) <- c("overall", "no college", "college", "no college reweighted", "college reweighted")
# 
# write.csv(balance_a, "table1.csv")
# write.csv(balance_b, "table2.csv")

# figure C1: balance before reweighting

figureC1_input <- Reduce(rbind, plot_b_before_ebal) %>%
  dplyr::select(college3, origin, fitted = response, se = SE) %>%
  mutate(cov = factor(rep(var2bal, each = 303),
                      levels = var_levels, labels = var_labels),
         Education = factor(college3, levels = 0:2,
                            labels = c("Full Sample",
                                       "Nonselective Colleges",
                                       "Selective Colleges")))
ggplot(figureC1_input, aes(x = origin, y = fitted)) +
  geom_line(aes(linetype = Education, col = Education), size = 1.1) +
  geom_ribbon(aes(ymin = fitted - 1.645*se, ymax = fitted + 1.645*se, fill = Education), alpha = 0.2) +
  facet_wrap(~cov, scales = "free_y", ncol = 3) +
  labs(x = "Parental Income Rank", y = "Conditonal Means of Covariates") +
  theme_minimal(base_size = 18) +
  theme(legend.position = "bottom") +
  guides(col = guide_legend(title = "", nrow = 1),
         fill = guide_legend(title = "", nrow = 1),
         linetype = guide_legend(title = "", nrow = 1)) 

ggsave("figureC1.tiff", width = 9, height = 9)

# figure C2: balance after reweighting

figureC2_input <- Reduce(rbind, plot_b_after_ebal) %>%
  dplyr::select(college3, origin, fitted = response, se = SE) %>%
  mutate(cov = factor(rep(var2bal, each = 303),
                      levels = var_levels, labels = var_labels),
         Education = factor(college3, levels = 0:2,
                            labels = c("Full Sample",
                                       "Nonselective Colleges",
                                       "Selective Colleges")))

ggplot(figureC2_input, aes(x = origin, y = fitted)) +
  geom_line(aes(linetype = Education, col = Education), size = 1.1) +
  geom_ribbon(aes(ymin = fitted - 1.645*se, ymax = fitted + 1.645*se, fill = Education), alpha = 0.2) +
  facet_wrap(~cov, scales = "free_y", ncol = 3) +
  labs(x = "Parental Income Rank", y = "Conditonal Means of Covariates") +
  theme_minimal(base_size = 18) +
  theme(legend.position = "bottom") +
  guides(col = guide_legend(title = "", nrow = 1),
         fill = guide_legend(title = "", nrow = 1),
         linetype = guide_legend(title = "", nrow = 1)) 

ggsave("figureC2.tiff", width = 9, height = 9)

# figure 6: conditional and controlled mobility

figure6_input <- rbind(msm_b_naive_plot, msm_b_ebal_plot) %>%
  dplyr::select(college3, origin, fitted = link, se = SE) %>%
  mutate(method = factor(rep(c("before", "after"), each = 303),
                         levels = c("before", "after"),
                         labels = c("Before Reweighting", "After Reweighting")),
         Education = factor(college3, levels = 0:2,
                            labels = c("Non-College Graduates",
                                       "Nonselective Colleges",
                                       "Selective Colleges")))

ggplot(figure6_input, aes(x = origin, y = fitted)) +
  geom_line(aes(linetype = Education, col = Education), size = 1.1) +
  geom_ribbon(aes(ymin = fitted - 1.645*se, ymax = fitted + 1.645*se, fill = Education), alpha = 0.2) +
  facet_wrap(~method, scales = "fixed", ncol = 2) +
  labs(x = "Parental Income Rank", y = "Adult Income Rank") +
  theme_minimal(base_size = 20) +
  theme(legend.position = "bottom") +
  guides(col = guide_legend(title = "", nrow = 1),
         fill = guide_legend(title = "", nrow = 1),
         linetype = guide_legend(title = "", nrow = 1)) 

ggsave("figure6.tiff", width = 9, height = 6)
# ggsave("figure6slides_b.tiff", width = 9, height = 6)

ggplot(figure6_input %>% mutate(fitted = ifelse(method == "After Reweighting", NA, fitted)), aes(x = origin, y = fitted)) +
  geom_line(aes(linetype = Education, col = Education), size = 1.1) +
  geom_ribbon(aes(ymin = fitted - 1.645*se, ymax = fitted + 1.645*se, fill = Education), alpha = 0.2) +
  facet_wrap(~method, scales = "fixed", ncol = 2) +
  labs(x = "Parental Income Rank", y = "Adult Income Rank") +
  theme_minimal(base_size = 20) +
  theme(legend.position = "bottom") +
  guides(col = guide_legend(title = "", nrow = 1),
         fill = guide_legend(title = "", nrow = 1),
         linetype = guide_legend(title = "", nrow = 1)) 

# ggsave("figure6slides_a.tiff", width = 9, height = 6)


