rm(list=ls(all=TRUE))

library("foreign")
library("Hmisc")
library("haven")
library("readr")
library("tidyr")
library("dplyr")
library("ggplot2")
library("gridExtra")
library("ltm")
library("pryr")
library("car")
library("xtable")
library("ebal")
library("twang")
library("mgcv")
library("survey")
library("splines")
source("01_star.R")

load("results_male_ipw.RData")
load("results_female_ipw.RData")
load("results_full_ipw.RData")

var_labels <- c("Female", "Black", "Hispanic", "Mother's Education", "Father Presence",
                "Number of Siblings", "Urban Residence", "Adjusted AFQT Score", "Educational Expectation") 
var_levels <- c("female", "black", "hispanic", "educ_mom", "father", "num_sibs", "urban", "test_score", "educ_exp")

nrow(nlsy79_ed3_b)
nrow(nlsy79_male_ed3_b)
nrow(nlsy79_female_ed3_b)

# table A1

coef <- list(msm_b_naive, msm_b_ipw,
             msm_b_naive_male, msm_b_ipw_male,
             msm_b_naive_female, msm_b_ipw_female) %>%
  lapply(summary) %>% lapply(coef) %>%  lapply(`[`, , 1) %>% simplify2array()

se <- list(msm_b_naive, msm_b_ipw,
           msm_b_naive_male, msm_b_ipw_male,
           msm_b_naive_female, msm_b_ipw_female) %>%
  lapply(summary) %>% lapply(coef) %>% lapply(`[`, , 2) %>% simplify2array()

comb(coef, se) %>% write.csv("tableA1.csv", row.names = FALSE)

# write.csv(coef, "tableA1coef_ipw.csv")
# write.csv(se, "tableA1se_ipw.csv")
# 
# list(msm_b_naive, msm_b_ipw,
#      msm_b_naive_male, msm_b_ipw_male,
#      msm_b_naive_female, msm_b_ipw_female) %>%
#   lapply(summary)

# figure 2: balance before reweighting

figure2_input <- Reduce(rbind, plot_b_before_ipw) %>%
  dplyr::select(college, origin, fitted = response, se = SE) %>%
  mutate(cov = factor(rep(var2bal, each = 202),
                      levels = var_levels, labels = var_labels),
         Education = factor(college, levels = c(F, T), labels = c("Full Sample", "College Graduates")))

ggplot(figure2_input, aes(x = origin, y = fitted)) +
  geom_line(aes(linetype = Education, col = Education), size = 1.1) +
  geom_ribbon(aes(ymin = fitted - 1.645*se, ymax = fitted + 1.645*se, fill = Education), alpha = 0.2) +
  facet_wrap(~cov, scales = "free_y", ncol = 3) +
  labs(x = "Parental Income Rank", y = "Conditonal Means of Covariates") +
  theme_minimal(base_size = 18) +
  theme(legend.position = "bottom") +
  guides(col = guide_legend(title = "", nrow = 1),
         fill = guide_legend(title = "", nrow = 1),
         linetype = guide_legend(title = "", nrow = 1)) 

# figure 3: balance after reweighting

figure3_input <- Reduce(rbind, plot_b_after_ipw) %>%
  dplyr::select(college, origin, fitted = response, se = SE) %>%
  mutate(cov = factor(rep(var2bal, each = 202),
                      levels = var_levels, labels = var_labels),
         Education = factor(college, levels = c(F, T), labels = c("Full Sample", "College Graduates")))

ggplot(figure3_input, aes(x = origin, y = fitted)) +
  geom_line(aes(linetype = Education, col = Education), size = 1.1) +
  geom_ribbon(aes(ymin = fitted - 1.645*se, ymax = fitted + 1.645*se, fill = Education), alpha = 0.2) +
  facet_wrap(~cov, scales = "free_y", ncol = 3) +
  labs(x = "Parental Income Rank", y = "Conditonal Means of Covariates") +
  theme_minimal(base_size = 18) +
  theme(legend.position = "bottom") +
  guides(col = guide_legend(title = "", nrow = 1),
         fill = guide_legend(title = "", nrow = 1),
         linetype = guide_legend(title = "", nrow = 1)) 

ggsave("figureA1.tiff", width = 9, height = 9)
