ResBal <- function(M, C, Q, Z = rep(0, ncol(C)), max_iter = 200, tol = 1, print.level = 1) {
  
  converged <- FALSE
  for (iter in 1:max_iter) {
    
    if (print.level >= 1) {
      cat("iteration ", iter, "\n")
    }
    
    W <-  c(Q * exp(C %*% Z))
    sumC   <- t(C) %*% W
    gradient   <- sumC - M
    
    if (max(abs(gradient)) < tol) {
      converged <- TRUE
      break
    }
    
    if (print.level >= 2) {
      cat("Iteration", iter, "maximum deviation is =", format(max(abs(gradient)), digits = 4), "\n")
    }
    
    hessian = t(C) %*% (W * C)
    Coefs <- Z
    newton <- solve(hessian, gradient)
    Z  <- Z - newton
    
    loss.new <- line.searcher(Base.weight = Q, Co.x = C, Tr.total = M, coefs = Z, Newton = newton, ss = 1)
    loss.old <- line.searcher(Base.weight = Q, Co.x = C, Tr.total = M, coefs = Coefs, Newton = newton, ss = 0)
    
    if (print.level >= 3) {cat("new loss", loss.new, "old loss=", loss.old, "\n")}
    
    if (loss.old <= loss.new) {
      
      ss.out <- optimize(line.searcher, lower = .00001, upper = 1, maximum = FALSE, Base.weight = Q,
        Co.x = C, Tr.total = M, coefs = Coefs, Newton = newton)
      
      if (print.level >= 3) {
        cat("LS Step Length is ", ss.out$minimum, "\n")
      }
      
      if (print.level >= 3) {
        cat("Loss is", ss.out$objective, "\n")
      }
      
      Z <- Coefs - ss.out$minimum * solve(hessian, gradient)
    }
  }
  list(W = W, sumC = sumC, converged = converged, maxdiff = max(abs(gradient)))
}
